@@ -11,8 +11,8 @@ class PyinfraError(Exception):
 
 class OperationError(PyinfraError):
     '''
-    Exception raised during fact gathering staging if an
-    operation is unable to generate output/change state.
+    Exception raised during fact gathering staging if an operation is unable to generate
+    output/change state.
     '''
 
 
@@ -20,3 +20,15 @@ class DeployError(PyinfraError):
     '''
     User exception for raising in deploys or sub deploys.
     '''
+
+
+class NoHostError(PyinfraError, TypeError):
+    '''
+    Raised when an inventory is missing a host.
+    '''
+
+
+class NoGroupError(PyinfraError, TypeError):
+    '''
+    Raise when an inventory is missing a group.
+    '''
