@@ -20,15 +20,10 @@ class PipPackages(FactBase):
     '''
 
     default = {}
-    command = 'pip freeze'
     _regex = r'^([a-zA-Z0-9_\-\+\.]+)==([0-9\.]+[a-z0-9\-]*)$'
 
+    def command(self, pip='pip'):
+        return '{0} freeze'.format(pip)
+
     def process(self, output):
         return parse_packages(self._regex, output)
-
-
-class PipVirtualenvPackages(PipPackages):
-    def command(self, venv):
-        # Remove any trailing slash
-        venv = venv.rstrip('/')
-        return '{0}/bin/pip freeze'.format(venv)
