@@ -9,8 +9,10 @@ The files module handles filesystem state, file uploads and template generation.
 from __future__ import unicode_literals
 
 import sys
-from os import path, walk
+
 from datetime import timedelta
+from fnmatch import fnmatch
+from os import path, walk
 
 import six
 from jinja2 import UndefinedError
@@ -22,11 +24,11 @@ from .util.files import chmod, chown, ensure_mode_int, sed_replace
 
 
 @operation(pipeline_facts={
-    'file': 'destination'
+    'file': 'destination',
 })
 def download(
     state, host, source_url, destination,
-    user=None, group=None, mode=None, cache_time=None, force=False
+    user=None, group=None, mode=None, cache_time=None, force=False,
 ):
     '''
     Download files from remote locations.
@@ -40,15 +42,13 @@ def download(
     + force: always download the file, even if it already exists
     '''
 
-    commands = []
-
     # Get destination info
     info = host.fact.file(destination)
 
     # Destination is a directory?
     if info is False:
         raise OperationError(
-            'Destination {0} already exists and is not a file'.format(destination)
+            'Destination {0} already exists and is not a file'.format(destination),
         )
 
     # Do we download the file? Force by default
@@ -69,15 +69,13 @@ def download(
 
     # If we download, always do user/group/mode as SSH user may be different
     if download:
-        commands.append('wget -q {0} -O {1}'.format(source_url, destination))
+        yield 'wget -q {0} -O {1}'.format(source_url, destination)
 
         if user or group:
-            commands.append(chown(destination, user, group))
+            yield chown(destination, user, group)
 
         if mode:
-            commands.append(chmod(destination, mode))
-
-    return commands
+            yield chmod(destination, mode)
 
 
 @operation
@@ -112,7 +110,6 @@ def line(state, host, name, line, present=True, replace=None, flags=None):
 
     # Is there a matching line in this file?
     present_lines = host.fact.find_in_file(name, match_line)
-    commands = []
 
     # If replace present, use that over the matching line
     if replace:
@@ -123,14 +120,17 @@ def line(state, host, name, line, present=True, replace=None, flags=None):
 
     # Save commands for re-use in dynamic script when file not present at fact stage
     echo_command = 'echo "{0}" >> {1}'.format(line, name)
-    sed_replace_command = sed_replace(state, name, match_line, replace, flags=flags)
+    sed_replace_command = sed_replace(
+        name, match_line, replace,
+        flags=flags,
+    )
 
     # No line and we want it, append it
     if not present_lines and present:
-        # If the file does not exist - it *might* be created, so we handle it dynamically
-        # with a little script.
+        # If the file does not exist - it *might* be created, so we handle it
+        # dynamically with a little script.
         if present_lines is None:
-            commands.append('''
+            yield '''
                 # If the file now exists
                 if [ -f "{target}" ]; then
                     # Grep for the line, sed if matches
@@ -146,24 +146,22 @@ def line(state, host, name, line, present=True, replace=None, flags=None):
                 target=name,
                 match_line=match_line,
                 echo_command=echo_command,
-                sed_replace_command=sed_replace_command
-            ))
+                sed_replace_command=sed_replace_command,
+            )
 
         # Otherwise the file exists and there is no matching line, so append it
         else:
-            commands.append(echo_command)
+            yield echo_command
 
     # Line(s) exists and we want to remove them, replace with nothing
     elif present_lines and not present:
-        commands.append(sed_replace(state, name, match_line, '', flags=flags))
+        yield sed_replace(name, match_line, '', flags=flags)
 
     # Line(s) exists and we have want to ensure they're correct
     elif present_lines and present:
         # If any of lines are different, sed replace them
         if replace and any(line != replace for line in present_lines):
-            commands.append(sed_replace_command)
-
-    return commands
+            yield sed_replace_command
 
 
 @operation
@@ -177,13 +175,16 @@ def replace(state, host, name, match, replace, flags=None):
     + flags: list of flaggs to pass to sed
     '''
 
-    return [sed_replace(state, name, match, replace, flags=flags)]
+    yield sed_replace(name, match, replace, flags=flags)
 
 
 @operation(pipeline_facts={
-    'find_files': 'destination'
+    'find_files': 'destination',
 })
-def sync(state, host, source, destination, user=None, group=None, mode=None, delete=False):
+def sync(
+    state, host, source, destination,
+    user=None, group=None, mode=None, delete=False, exclude=None,
+):
     '''
     Syncs a local directory with a remote one, with delete support. Note that delete will
     remove extra files on the remote side, but not extra directories.
@@ -194,6 +195,7 @@ def sync(state, host, source, destination, user=None, group=None, mode=None, del
     + group: group to own the files and directories
     + mode: permissions of the files
     + delete: delete remote files not present locally
+    + exclude: string or list/tuple of strings to match & exclude files (eg *.pyc)
     '''
 
     # If we don't enforce the source ending with /, remote_dirname below might start with
@@ -205,6 +207,11 @@ def sync(state, host, source, destination, user=None, group=None, mode=None, del
     if state.deploy_dir:
         source = path.join(state.deploy_dir, source)
 
+    # Ensure exclude is a list/tuple
+    if exclude is not None:
+        if not isinstance(exclude, (list, tuple)):
+            exclude = [exclude]
+
     put_files = []
     ensure_dirnames = []
     for dirname, _, filenames in walk(source):
@@ -214,41 +221,51 @@ def sync(state, host, source, destination, user=None, group=None, mode=None, del
             ensure_dirnames.append(remote_dirname)
 
         for filename in filenames:
+            full_filename = path.join(dirname, filename)
+
+            # Should we exclude this file?
+            to_exclude = False
+            if exclude:
+                for match in exclude:
+                    if fnmatch(full_filename, match):
+                        to_exclude = True
+
+            if to_exclude:
+                continue
+
             put_files.append((
                 # Join local as normal (unix, win)
-                path.join(dirname, filename),
+                full_filename,
                 # Join remote as unix like
                 '/'.join(
                     item for item in
                     (destination, remote_dirname, filename)
                     if item
-                )
+                ),
             ))
 
-    commands = []
-
     # Ensure the destination directory
-    commands.extend(directory(
+    yield directory(
         state, host, destination,
-        user=user, group=group
-    ))
+        user=user, group=group,
+    )
 
     # Ensure any remote dirnames
     for dirname in ensure_dirnames:
-        commands.extend(directory(
+        yield directory(
             state, host,
             '/'.join((destination, dirname)),
-            user=user, group=group
-        ))
+            user=user, group=group,
+        )
 
     # Put each file combination
     for local_filename, remote_filename in put_files:
-        commands.extend(put(
+        yield put(
             state, host,
             local_filename, remote_filename,
             user=user, group=group, mode=mode,
-            add_deploy_dir=False
-        ))
+            add_deploy_dir=False,
+        )
 
     # Delete any extra files
     if delete:
@@ -256,18 +273,16 @@ def sync(state, host, source, destination, user=None, group=None, mode=None, del
         wanted_filenames = set([remote_filename for _, remote_filename in put_files])
         files_to_delete = remote_filenames - wanted_filenames
         for filename in files_to_delete:
-            commands.extend(file(state, host, filename, present=False))
-
-    return commands
+            yield file(state, host, filename, present=False)
 
 
 @operation(pipeline_facts={
     'file': 'remote_filename',
-    'sha1_file': 'remote_filename'
+    'sha1_file': 'remote_filename',
 })
 def put(
     state, host, local_filename, remote_filename,
-    user=None, group=None, mode=None, add_deploy_dir=True
+    user=None, group=None, mode=None, add_deploy_dir=True,
 ):
     '''
     Copy a local file to the remote system.
@@ -296,17 +311,16 @@ def put(
 
     mode = ensure_mode_int(mode)
     remote_file = host.fact.file(remote_filename)
-    commands = []
 
     # No remote file, always upload and user/group/mode if supplied
     if not remote_file:
-        commands.append((local_file, remote_filename))
+        yield (local_file, remote_filename)
 
         if user or group:
-            commands.append(chown(remote_filename, user, group))
+            yield chown(remote_filename, user, group)
 
         if mode:
-            commands.append(chmod(remote_filename, mode))
+            yield chmod(remote_filename, mode)
 
     # File exists, check sum and check user/group/mode if supplied
     else:
@@ -315,27 +329,25 @@ def put(
 
         # Check sha1sum, upload if needed
         if local_sum != remote_sum:
-            commands.append((local_file, remote_filename))
+            yield (local_file, remote_filename)
 
             if user or group:
-                commands.append(chown(remote_filename, user, group))
+                yield chown(remote_filename, user, group)
 
             if mode:
-                commands.append(chmod(remote_filename, mode))
+                yield chmod(remote_filename, mode)
 
         else:
             # Check mode
             if mode and remote_file['mode'] != mode:
-                commands.append(chmod(remote_filename, mode))
+                yield chmod(remote_filename, mode)
 
             # Check user/group
             if (
                 (user and remote_file['user'] != user)
                 or (group and remote_file['group'] != group)
             ):
-                commands.append(chown(remote_filename, user, group))
-
-    return commands
+                yield chown(remote_filename, user, group)
 
 
 @operation
@@ -385,27 +397,27 @@ def template(
         relevant_lines = template_lines[max(line_number - 2, 0):line_number + 1]
 
         raise OperationError('Error in template: {0} (L{1}): {2}\n...\n{3}\n...'.format(
-            template_filename, line_number, e, '\n'.join(relevant_lines)
+            template_filename, line_number, e, '\n'.join(relevant_lines),
         ))
 
     output_file = six.StringIO(output)
     output_file.template = template_filename
 
     # Pass to the put function
-    return put(
+    yield put(
         state, host,
         output_file, remote_filename,
         user=user, group=group, mode=mode,
-        add_deploy_dir=False
+        add_deploy_dir=False,
     )
 
 
 @operation(pipeline_facts={
-    'link': 'name'
+    'link': 'name',
 })
 def link(
     state, host, name,
-    target=None, present=True, user=None, group=None, symbolic=True
+    target=None, present=True, user=None, group=None, symbolic=True,
 ):
     '''
     Manage the state of links.
@@ -426,7 +438,6 @@ def link(
         raise OperationError('If present is True target must be provided')
 
     info = host.fact.link(name)
-    commands = []
 
     # Not a link?
     if info is False:
@@ -434,20 +445,20 @@ def link(
 
     add_cmd = 'ln{0} {1} {2}'.format(
         ' -s' if symbolic else '',
-        target, name
+        target, name,
     )
 
     remove_cmd = 'rm -f {0}'.format(name)
 
     # No link and we want it
     if info is None and present:
-        commands.append(add_cmd)
+        yield add_cmd
         if user or group:
-            commands.append(chown(name, user, group, dereference=False))
+            yield chown(name, user, group, dereference=False)
 
     # It exists and we don't want it
     elif info and not present:
-        commands.append(remove_cmd)
+        yield remove_cmd
 
     # Exists and want to ensure it's state
     elif info and present:
@@ -461,29 +472,25 @@ def link(
 
             if not path.isabs(info['link_target']):
                 info['link_target'] = path.normpath(
-                    '/'.join((link_dirname, info['link_target']))
+                    '/'.join((link_dirname, info['link_target'])),
                 )
 
         # If the target is wrong, remove & recreate the link
         if info['link_target'] != target:
-            commands.extend((
-                remove_cmd,
-                add_cmd
-            ))
+            yield remove_cmd
+            yield add_cmd
 
         # Check user/group
         if (user and info['user'] != user) or (group and info['group'] != group):
-            commands.append(chown(name, user, group, dereference=False))
-
-    return commands
+            yield chown(name, user, group, dereference=False)
 
 
 @operation(pipeline_facts={
-    'file': 'name'
+    'file': 'name',
 })
 def file(
     state, host, name,
-    present=True, user=None, group=None, mode=None, touch=False
+    present=True, user=None, group=None, mode=None, touch=False,
 ):
     '''
     Manage the state of files.
@@ -498,7 +505,6 @@ def file(
 
     mode = ensure_mode_int(mode)
     info = host.fact.file(name)
-    commands = []
 
     # Not a file?!
     if info is False:
@@ -506,39 +512,37 @@ def file(
 
     # Doesn't exist & we want it
     if info is None and present:
-        commands.append('touch {0}'.format(name))
+        yield 'touch {0}'.format(name)
 
         if mode:
-            commands.append(chmod(name, mode))
+            yield chmod(name, mode)
         if user or group:
-            commands.append(chown(name, user, group))
+            yield chown(name, user, group)
 
     # It exists and we don't want it
     elif info and not present:
-        commands.append('rm -f {0}'.format(name))
+        yield 'rm -f {0}'.format(name)
 
     # It exists & we want to ensure its state
     elif info and present:
         if touch:
-            commands.append('touch {0}'.format(name))
+            yield 'touch {0}'.format(name)
 
         # Check mode
         if mode and info['mode'] != mode:
-            commands.append(chmod(name, mode))
+            yield chmod(name, mode)
 
         # Check user/group
         if (user and info['user'] != user) or (group and info['group'] != group):
-            commands.append(chown(name, user, group))
-
-    return commands
+            yield chown(name, user, group)
 
 
 @operation(pipeline_facts={
-    'directory': 'name'
+    'directory': 'name',
 })
 def directory(
     state, host, name,
-    present=True, user=None, group=None, mode=None, recursive=False
+    present=True, user=None, group=None, mode=None, recursive=False,
 ):
     '''
     Manage the state of directories.
@@ -553,7 +557,6 @@ def directory(
 
     mode = ensure_mode_int(mode)
     info = host.fact.directory(name)
-    commands = []
 
     # Not a directory?!
     if info is False:
@@ -561,24 +564,22 @@ def directory(
 
     # Doesn't exist & we want it
     if info is None and present:
-        commands.append('mkdir -p {0}'.format(name))
+        yield 'mkdir -p {0}'.format(name)
         if mode:
-            commands.append(chmod(name, mode, recursive=recursive))
+            yield chmod(name, mode, recursive=recursive)
         if user or group:
-            commands.append(chown(name, user, group, recursive=recursive))
+            yield chown(name, user, group, recursive=recursive)
 
     # It exists and we don't want it
     elif info and not present:
-        commands.append('rm -rf {0}'.format(name))
+        yield 'rm -rf {0}'.format(name)
 
     # It exists & we want to ensure its state
     elif info and present:
         # Check mode
         if mode and info['mode'] != mode:
-            commands.append(chmod(name, mode, recursive=recursive))
+            yield chmod(name, mode, recursive=recursive)
 
         # Check user/group
         if (user and info['user'] != user) or (group and info['group'] != group):
-            commands.append(chown(name, user, group, recursive=recursive))
-
-    return commands
+            yield chown(name, user, group, recursive=recursive)
