@@ -21,14 +21,14 @@ def packages(state, host, packages=None, present=True, latest=False):
     + latest: whether to upgrade packages without a specified version
 
     Versions:
-        Package versions can be pinned like gem: ``<pkg>:<version>``
+        Package versions can be pinned like gem: ``<pkg>:<version>``.
     '''
 
-    return ensure_packages(
+    yield ensure_packages(
         packages, host.fact.gem_packages, present,
-        install_command='gem instal',
+        install_command='gem install',
         uninstall_command='gem uninstall',
-        version_join=':',
         upgrade_command='gem update',
-        latest=latest
+        version_join=':',
+        latest=latest,
     )
