@@ -20,14 +20,10 @@ def packages(state, host, packages=None, present=True, latest=False, directory=N
     + directory: directory to manage packages for, defaults to global
 
     Versions:
-        Package versions can be pinned like npm: ``<pkg>@<version>``
+        Package versions can be pinned like npm: ``<pkg>@<version>``.
     '''
 
-    current_packages = (
-        host.fact.npm_packages
-        if directory is None
-        else host.fact.npm_local_packages(directory)
-    )
+    current_packages = host.fact.npm_packages(directory)
 
     install_command = (
         'npm install -g'
@@ -47,11 +43,11 @@ def packages(state, host, packages=None, present=True, latest=False, directory=N
         else 'cd {0} && npm update'.format(directory)
     )
 
-    return ensure_packages(
+    yield ensure_packages(
         packages, current_packages, present,
         install_command=install_command,
         uninstall_command=uninstall_command,
-        version_join='@',
         upgrade_command=upgrade_command,
-        latest=latest
+        version_join='@',
+        latest=latest,
     )
