@@ -6,6 +6,10 @@ VERSION=`python setup.py --version`
 
 echo "# Releasing pyinfra v$VERSION..."
 
+echo "# Checking pandoc installed..."
+pandoc --help >/dev/null 2>&1 || (echo "pandoc is not installed!" && exit 1)
+python -c 'import pypandoc' >/dev/null 2>&1 || (echo "pypandoc is not installed!" && exit 1)
+
 echo "# Running tests..."
 nosetests -s
 
@@ -14,7 +18,6 @@ scripts/build_docs.sh
 
 echo "# Commit & push the docs..."
 git add docs/
-git add README.md
 git commit -m "Documentation update for v$VERSION." || echo "No docs updated!"
 git push
 
@@ -23,6 +26,11 @@ git tag -a "v$VERSION" -m "v$VERSION"
 git push --tags
 
 echo "# Upload to pypi..."
-python setup.py sdist bdist_wheel upload
+# Clear build & dist
+rm -rf build/* dist/*
+# Build source and wheel packages
+python setup.py sdist bdist_wheel
+# Upload w/Twine
+twine upload dist/*
 
 echo "# All done!"
