@@ -17,6 +17,9 @@ class FakeChannel(object):
     def exit_status_ready(self):
         return True
 
+    def recv_exit_status(self):
+        return self.exit_status
+
 
 class FakeBuffer(object):
     def __init__(self, data, channel):
@@ -41,7 +44,7 @@ class FakeSSHClient(object):
     def open_session(self):
         pass
 
-    def exec_command(self, command):
+    def exec_command(self, command, get_pty=None):
         channel = FakeChannel(0)
         return (
             channel,
